/* Copyright (C) 2001-2003 Peter Selinger.
   This file is part of potrace. It is free software and it is covered
   by the GNU general public license. See the file COPYING for details. */

/* $Id: gauss.c,v 1.2 2002/12/21 05:47:22 selinger Exp $ */

#include <math.h>

/* Dumb Gaussian elimination to solve matrix equation A*x=b.
   Destructively manipulates A and b. Not very efficient. Ap must
   point to an object of type double[n][n]. Returns 0 if the matrix
   was non-singular, 1 if it was singular and the solution was not
   unique, and -1 if it was singular and there was no solution. */

int solve(int n, void *Ap, double b[], double x[]) {
  int r[n];         /* indexing rows for quick reordering */
  int i, j, k, tmp, ret;
  double m, b0, d;
  double (*A)[n] = (double (*)[]) Ap;

  for (i=0; i<n; i++) {
    r[i]=i;
  }

  for (j=0; j<n-1; j++) {    /* make j'th column zero from j+1'st row */
    for (i=j+1; i<n; i++) {  /* make Aij zero */
      if (fabs(A[r[j]][j]) < fabs(A[r[i]][j])) {
	tmp=r[i]; 
	r[i]=r[j]; 
	r[j]=tmp;
      }
      if (A[r[i]][j] != 0.0) {
	m = A[r[i]][j] / A[r[j]][j];
	for (k=j; k<n; k++) {
	  A[r[i]][k] -= m*A[r[j]][k];
	} 
	b[r[i]] -= m*b[r[j]];
      }
    }
  }
  
  ret = 0;

  /* calculate result vectors */
  for (j=n-1; j>=0; j--) {
    b0 = b[r[j]];
    for (k=j+1; k<n; k++) {
      b0 -= x[k] * A[r[j]][k];
    }
    d = A[r[j]][j];
    if (d == 0.0 && b0 != 0.0) {
      return -1;
    } else if (d == 0.0) {
      x[j] = 0.0;
      ret = 1;
    } else {
      x[j] = b0 / A[r[j]][j];
    }
  }
  return ret;
}
