/* Copyright (C) 2001-2003 Peter Selinger.
   This file is part of potrace. It is free software and it is covered
   by the GNU general public license. See the file COPYING for details. */

/* $Id: pbm.h,v 1.5 2003/08/18 02:48:45 selinger Exp $ */

#ifndef PBM_H
#define PBM_H

#include <stdio.h>

typedef unsigned int word;
#define WORDSIZE sizeof(word)
#define WORDBITS (8*WORDSIZE)
#define HIBIT ((word)(1<<(WORDBITS-1)))

/* internal format for bitmaps. Note: in this format, rows are ordered
   from bottom to top. The bits in each row are divided into words
   from left to right, with the highest-valued bits on the left. */

struct bitmap_s {
  int w;                 /* width, in pixels */
  int h;                 /* height, in pixels */
  int c;                 /* ceil(w/WORDBITS) = words per row */
  word *map;             /* raw data, c*h words */
};
typedef struct bitmap_s bitmap_t;

/* macros for accessing pixel at index (x,y). Note that the origin is
   in the *lower* left corner. U* macros omit the bounds check. */

#define bm_index(bm, x, y) (&(bm)->map[(x)/WORDBITS+(y)*(bm)->c])
#define bm_mask(x) (HIBIT>>((x)%WORDBITS))
#define bm_safe(bm, x, y) ((x)>=0 && (x)<(bm)->w && (y)>=0 && (y)<(bm)->h)
#define BM_UGET(bm, x, y) (*bm_index(bm, x, y) & bm_mask(x))
#define BM_USET(bm, x, y) (*bm_index(bm, x, y) |= bm_mask(x))
#define BM_UCLR(bm, x, y) (*bm_index(bm, x, y) &= ~bm_mask(x))
#define BM_UINV(bm, x, y) (*bm_index(bm, x, y) ^= bm_mask(x))
#define BM_UPUT(bm, x, y, b) (b ? BM_USET(bm, x, y) : BM_UCLR(bm, x, y))
#define BM_GET(bm, x, y) (bm_safe(bm, x, y) ? BM_UGET(bm, x, y) : 0)
#define BM_SET(bm, x, y) (bm_safe(bm, x, y) ? BM_USET(bm, x, y) : 0)
#define BM_CLR(bm, x, y) (bm_safe(bm, x, y) ? BM_UCLR(bm, x, y) : 0)
#define BM_INV(bm, x, y) (bm_safe(bm, x, y) ? BM_UINV(bm, x, y) : 0)
#define BM_PUT(bm, x, y, b) (bm_safe(bm, x, y) ? BM_UPUT(bm, x, y, b) : 0)

extern char *bm_read_error;

bitmap_t *bm_new(int w, int h);
bitmap_t *bm_dup(bitmap_t *bm);
void bm_free(bitmap_t *bm);
void bm_clear(bitmap_t *bm, int c);
void bm_invert(bitmap_t *bm);
int bm_read(FILE *f, double blacklevel, bitmap_t **bmp);
int bm_print(FILE *f, bitmap_t *bm);

#endif /* PBM_H */

